package net.knarcraft.knarlib.formatting;

import net.knarcraft.knarlib.property.ColorConversion;
import net.knarcraft.knarlib.util.ColorHelper;
import net.md_5.bungee.api.ChatColor;
import org.bukkit.command.CommandSender;

/**
 * A formatter for formatting displayed messages
 */
@SuppressWarnings("unused")
public final class StringFormatter {

    private final String pluginName;
    private final Translator translator;
    private ChatColor successColor = ChatColor.GREEN;
    private ChatColor errorColor = ChatColor.DARK_RED;

    /**
     * Instantiates a new string formatter
     *
     * <p>You may leave one of these as null if not in use, for example if you don't use translatable strings.</p>
     *
     * @param pluginName <p>The name of the using plugin (used for the message prefix)</p>
     * @param translator <p>The translator to use for translating messages</p>
     */
    public StringFormatter(String pluginName, Translator translator) {
        this.pluginName = pluginName;
        this.translator = translator;
    }

    /**
     * Sets the color to prepend when using displaySuccessMessage
     *
     * @param color <p>The color to use</p>
     */
    public void setSuccessColor(ChatColor color) {
        if (color != null) {
            this.successColor = color;
        } else {
            throw new IllegalArgumentException("Success color cannot be null");
        }
    }

    /**
     * Sets the color to prepend when using displayErrorMessage
     *
     * @param color <p>The color to use</p>
     */
    public void setErrorColor(ChatColor color) {
        if (color != null) {
            this.errorColor = color;
        } else {
            throw new IllegalArgumentException("Error color cannot be null");
        }
    }

    /**
     * Displays a formatted message
     *
     * @param sender  <p>The command sender to display the message to</p>
     * @param message <p>The translatable message to display</p>
     */
    public void displayNeutralMessage(CommandSender sender, TranslatableMessage message) {
        sender.sendMessage(getFormattedMessage(this.translator.getTranslatedMessage(message)));
    }

    /**
     * Displays a formatted message
     *
     * @param sender  <p>The command sender to display the message to</p>
     * @param message <p>The raw message to display</p>
     */
    public void displayNeutralMessage(CommandSender sender, String message) {
        sender.sendMessage(getFormattedMessage(message));
    }

    /**
     * Displays a message signifying a successful action
     *
     * @param sender  <p>The command sender to display the message to</p>
     * @param message <p>The translatable message to display</p>
     */
    public void displaySuccessMessage(CommandSender sender, TranslatableMessage message) {
        sender.sendMessage(successColor + getFormattedMessage(this.translator.getTranslatedMessage(message)));
    }

    /**
     * Displays a message signifying a successful action
     *
     * @param sender  <p>The command sender to display the message to</p>
     * @param message <p>The raw message to display</p>
     */
    public void displaySuccessMessage(CommandSender sender, String message) {
        sender.sendMessage(successColor + getFormattedMessage(message));
    }

    /**
     * Displays a message signifying an unsuccessful action
     *
     * @param sender  <p>The command sender to display the message to</p>
     * @param message <p>The translatable message to display</p>
     */
    public void displayErrorMessage(CommandSender sender, TranslatableMessage message) {
        sender.sendMessage(errorColor + getFormattedMessage(this.translator.getTranslatedMessage(message)));
    }

    /**
     * Displays a message signifying an unsuccessful action
     *
     * @param sender  <p>The command sender to display the message to</p>
     * @param message <p>The raw message to display</p>
     */
    public void displayErrorMessage(CommandSender sender, String message) {
        sender.sendMessage(errorColor + getFormattedMessage(message));
    }

    /**
     * Replaces a placeholder in a translatable message
     *
     * @param translatableMessage <p>The translatable message to replace in</p>
     * @param placeholder         <p>The placeholder to replace</p>
     * @param replacement         <p>The replacement value</p>
     * @return <p>The input string with the placeholder replaced</p>
     */
    public String replacePlaceholder(TranslatableMessage translatableMessage, String placeholder, String replacement) {
        return replacePlaceholder(this.translator.getTranslatedMessage(translatableMessage), placeholder, replacement);
    }

    /**
     * Replaces placeholders in a translatable message
     *
     * @param translatableMessage <p>The translatable message to replace in</p>
     * @param placeholders        <p>The placeholders to replace</p>
     * @param replacements        <p>The replacement values</p>
     * @return <p>The input string with placeholders replaced</p>
     */
    public String replacePlaceholders(TranslatableMessage translatableMessage, String[] placeholders,
                                      String[] replacements) {
        return replacePlaceholders(this.translator.getTranslatedMessage(translatableMessage), placeholders, replacements);
    }

    /**
     * Replaces a placeholder in a string
     *
     * @param input       <p>The input string to replace in</p>
     * @param placeholder <p>The placeholder to replace</p>
     * @param replacement <p>The replacement value</p>
     * @return <p>The input string with the placeholder replaced</p>
     */
    public static String replacePlaceholder(String input, String placeholder, String replacement) {
        return input.replace(placeholder, replacement);
    }

    /**
     * Replaces placeholders in a string
     *
     * @param input        <p>The input string to replace in</p>
     * @param placeholders <p>The placeholders to replace</p>
     * @param replacements <p>The replacement values</p>
     * @return <p>The input string with placeholders replaced</p>
     */
    public static String replacePlaceholders(String input, String[] placeholders, String[] replacements) {
        for (int i = 0; i < Math.min(placeholders.length, replacements.length); i++) {
            input = replacePlaceholder(input, placeholders[i], replacements[i]);
        }
        return input;
    }

    /**
     * Gets the formatted version of any chat message
     *
     * @param message <p>The message to format</p>
     * @return <p>The formatted message</p>
     */
    private String getFormattedMessage(String message) {
        message = ColorHelper.translateColorCodes(message, ColorConversion.NORMAL);
        if (this.pluginName == null) {
            return message;
        } else {
            return "[" + this.pluginName + "] " + ChatColor.RESET + message;
        }
    }

}
